@@ -1,13 +1,19 @@
 #![cfg(target_pointer_width = "64")]
 
+pub mod iter;
+
 use std::io::{Read, Write};
 use std::mem::size_of;
+use std::ops::Range;
 
 use anyhow::{anyhow, Result};
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
 
+use crate::elias_fano::iter::EliasFanoIterator;
 use crate::{broadword, darray::DArrayIndex, BitVector};
 
+const LINEAR_SCAN_THRESHOLD: usize = 64;
+
 /// Compressed monotone increasing sequence through Elias-Fano encoding.
 ///
 /// [`EliasFano`] implements an Elias-Fano representation for monotone increasing sequences.
@@ -159,7 +165,7 @@ impl EliasFano {
         let n = bv.len();
         let m = (0..bv.num_words())
             .into_iter()
-            .fold(0, |acc, i| acc + broadword::popcount(bv.get_word(i)));
+            .fold(0, |acc, i| acc + broadword::popcount(bv.words()[i]));
         let mut b = EliasFanoBuilder::new(n, m)?;
         for i in 0..n {
             if bv.get_bit(i) {
@@ -169,6 +175,37 @@ impl EliasFano {
         Ok(Self::new(b, with_rank_index))
     }
 
+    /// Creates a new [`EliasFano`] from monotone-increased integers.
+    ///
+    /// # Arguments
+    ///
+    /// - `ints`: Slice of monotone-increased integers .
+    /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
+    ///
+    /// # Errors
+    ///
+    /// `anyhow::Error` will be returned if `ints` is empty or is not monotone-increased.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::EliasFano;
+    ///
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// assert_eq!(ef.select(0), 1);
+    /// assert_eq!(ef.select(1), 3);
+    /// assert_eq!(ef.select(2), 3);
+    /// assert_eq!(ef.select(3), 7);
+    /// ```
+    pub fn from_ints(ints: &[usize], with_rank_index: bool) -> Result<Self> {
+        if ints.is_empty() {
+            return Err(anyhow!("The input ints must not be empty."));
+        }
+        let mut b = EliasFanoBuilder::new(*ints.last().unwrap() + 1, ints.len())?;
+        b.append(ints)?;
+        Ok(Self::new(b, with_rank_index))
+    }
+
     /// Serializes the data structure into the writer,
     /// returning the number of serialized bytes.
     ///
@@ -245,9 +282,11 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true], false).unwrap();
-    /// assert_eq!(ef.select(0), 0);
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// assert_eq!(ef.select(0), 1);
     /// assert_eq!(ef.select(1), 3);
+    /// assert_eq!(ef.select(2), 3);
+    /// assert_eq!(ef.select(3), 7);
     /// ```
     #[inline(always)]
     pub fn select(&self, k: usize) -> usize {
@@ -270,11 +309,11 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true], true).unwrap();
-    /// assert_eq!(ef.rank(1), 1);
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], true).unwrap();
+    /// assert_eq!(ef.rank(1), 0);
     /// assert_eq!(ef.rank(2), 1);
     /// assert_eq!(ef.rank(3), 1);
-    /// assert_eq!(ef.rank(4), 2);
+    /// assert_eq!(ef.rank(4), 3);
     /// ```
     #[inline(always)]
     pub fn rank(&self, pos: usize) -> usize {
@@ -320,9 +359,10 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[false, true, false, false, true], true).unwrap();
-    /// assert_eq!(ef.predecessor(4), Some(4));
-    /// assert_eq!(ef.predecessor(3), Some(1));
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], true).unwrap();
+    /// assert_eq!(ef.predecessor(4), Some(3));
+    /// assert_eq!(ef.predecessor(3), Some(3));
+    /// assert_eq!(ef.predecessor(2), Some(1));
     /// assert_eq!(ef.predecessor(0), None);
     /// ```
     #[inline(always)]
@@ -347,10 +387,11 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true, false], true).unwrap();
-    /// assert_eq!(ef.successor(0), Some(0));
-    /// assert_eq!(ef.successor(1), Some(3));
-    /// assert_eq!(ef.successor(4), None);
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], true).unwrap();
+    /// assert_eq!(ef.successor(0), Some(1));
+    /// assert_eq!(ef.successor(2), Some(3));
+    /// assert_eq!(ef.successor(3), Some(3));
+    /// assert_eq!(ef.successor(8), None);
     /// ```
     #[inline(always)]
     pub fn successor(&self, pos: usize) -> Option<usize> {
@@ -375,9 +416,11 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[false, true, false, false, true], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
     /// assert_eq!(ef.delta(0), 1);
-    /// assert_eq!(ef.delta(1), 3);
+    /// assert_eq!(ef.delta(1), 2);
+    /// assert_eq!(ef.delta(2), 0);
+    /// assert_eq!(ef.delta(3), 4);
     /// ```
     #[inline(always)]
     pub fn delta(&self, k: usize) -> usize {
@@ -392,6 +435,110 @@ impl EliasFano {
         }
     }
 
+    /// Finds the position `k` such that `select(k) == val`.
+    ///
+    /// Note that, if there are multiple values of `val`, one of them is returned.
+    ///
+    /// # Arguments
+    ///
+    /// - `val`: Integer to be searched.
+    ///
+    /// # Complexity
+    ///
+    /// - Logarithmic
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::EliasFano;
+    ///
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10], false).unwrap();
+    /// assert_eq!(ef.find(6), Some(3));
+    /// assert_eq!(ef.find(10), Some(5));
+    /// assert_eq!(ef.find(9), None);
+    /// ```
+    #[inline(always)]
+    pub fn find(&self, val: usize) -> Option<usize> {
+        self.find_range(0..self.len(), val)
+    }
+
+    /// Finds the position `k` such that `select(k) == val` and `k in range`.
+    ///
+    /// Note that, if there are multiple values of `val`, one of them is returned.
+    ///
+    /// # Arguments
+    ///
+    /// - `range`: Position range to be searched.
+    /// - `val`: Integer to be searched.
+    ///
+    /// # Complexity
+    ///
+    /// - Logarithmic for the range
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::EliasFano;
+    ///
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10], false).unwrap();
+    /// assert_eq!(ef.find_range(1..4, 6), Some(3));
+    /// assert_eq!(ef.find_range(5..6, 10), Some(5));
+    /// assert_eq!(ef.find_range(1..3, 6), None);
+    /// ```
+    #[inline(always)]
+    pub fn find_range(&self, range: Range<usize>, val: usize) -> Option<usize> {
+        if range.is_empty() {
+            return None;
+        }
+
+        // Binary search
+        let (mut lo, mut hi) = (range.start, range.end);
+        while hi - lo > LINEAR_SCAN_THRESHOLD {
+            let mi = (lo + hi) / 2;
+            let x = self.select(mi);
+            if val == x {
+                return Some(mi);
+            }
+            if val < x {
+                hi = mi;
+            } else {
+                lo = mi + 1;
+            }
+        }
+
+        // Linear scan
+        let mut it = self.iter(lo);
+        for i in lo..hi {
+            let x = it.next().unwrap();
+            if val == x {
+                return Some(i);
+            }
+        }
+        None
+    }
+
+    /// Creates an iterator of [`EliasFanoIterator`] to enumerate integers from the `k`-th one.
+    ///
+    /// # Arguments
+    ///
+    /// - `k`: Select query.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::EliasFano;
+    ///
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// let mut it = ef.iter(1);
+    /// assert_eq!(it.next(), Some(3));
+    /// assert_eq!(it.next(), Some(3));
+    /// assert_eq!(it.next(), Some(7));
+    /// assert_eq!(it.next(), None);
+    /// ```
+    pub fn iter(&self, k: usize) -> EliasFanoIterator {
+        EliasFanoIterator::new(self, k)
+    }
+
     /// Gets the number of integers.
     #[inline(always)]
     pub const fn len(&self) -> usize {
@@ -548,9 +695,50 @@ mod tests {
     use rand::{Rng, SeedableRng};
     use rand_chacha::ChaChaRng;
 
-    fn gen_random_bits(len: usize, seed: u64) -> Vec<bool> {
+    fn gen_random_bits(len: usize, p: f64, seed: u64) -> Vec<bool> {
         let mut rng = ChaChaRng::seed_from_u64(seed);
-        (0..len).map(|_| rng.gen::<bool>()).collect()
+        (0..len).map(|_| rng.gen_bool(p)).collect()
+    }
+
+    fn gen_random_queries(bits: &[bool], len: usize, seed: u64) -> Vec<(usize, Option<usize>)> {
+        let mut rng = ChaChaRng::seed_from_u64(seed);
+        let mut queries = vec![];
+        for _ in 0..len {
+            let v = rng.gen_range(0..bits.len());
+            queries.push(if bits[v] {
+                let rank = bits[..v].iter().fold(0, |acc, &b| acc + b as usize);
+                (v, Some(rank))
+            } else {
+                (v, None)
+            })
+        }
+        queries
+    }
+
+    fn gen_random_range_queries(
+        bits: &[bool],
+        len: usize,
+        seed: u64,
+    ) -> Vec<(usize, Range<usize>, Option<usize>)> {
+        let ints: Vec<usize> = bits
+            .iter()
+            .enumerate()
+            .filter(|(_, &b)| b)
+            .map(|(i, _)| i)
+            .collect();
+        let mut rng = ChaChaRng::seed_from_u64(seed);
+        let mut queries = vec![];
+        for _ in 0..len {
+            let v = rng.gen_range(0..bits.len());
+            let i = rng.gen_range(0..ints.len());
+            let j = rng.gen_range(i..ints.len());
+            queries.push(if let Some(p) = ints[i..j].iter().position(|&x| x == v) {
+                (v, i..j, Some(p + i))
+            } else {
+                (v, i..j, None)
+            })
+        }
+        queries
     }
 
     fn test_rank_select(bits: &[bool], ef: &EliasFano) {
@@ -590,30 +778,86 @@ mod tests {
         debug_assert!(pos == ef.universe() || ef.predecessor(pos).is_none());
     }
 
-    #[test]
-    fn test_tiny_bits() {
-        let ef =
-            EliasFano::from_bits(&[true, false, false, true, false, true, true], false).unwrap();
-        assert_eq!(ef.select(0), 0);
-        assert_eq!(ef.select(1), 3);
-        assert_eq!(ef.select(2), 5);
-        assert_eq!(ef.select(3), 6);
+    fn test_find(ef: &EliasFano, queries: &[(usize, Option<usize>)]) {
+        for &(val, expected) in queries {
+            assert_eq!(ef.find(val), expected);
+        }
     }
 
-    #[test]
-    fn test_random_bits() {
-        for seed in 0..100 {
-            let bits = gen_random_bits(10000, seed);
-            let ef = EliasFano::from_bits(&bits, true).unwrap();
-            test_rank_select(&bits, &ef);
-            test_successor_predecessor(&bits, &ef);
+    fn test_find_range(ef: &EliasFano, queries: &[(usize, Range<usize>, Option<usize>)]) {
+        for (val, rng, expected) in queries.iter().cloned() {
+            assert_eq!(ef.find_range(rng, val), expected);
         }
     }
 
     #[test]
-    fn test_serialize() {
+    fn test_tiny_bits() {
+        let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10], false).unwrap();
+        assert_eq!(ef.select(0), 1);
+        assert_eq!(ef.select(1), 3);
+        assert_eq!(ef.select(2), 3);
+        assert_eq!(ef.select(3), 6);
+        assert_eq!(ef.select(4), 7);
+        assert_eq!(ef.select(5), 10);
+
+        assert_eq!(ef.find(1), Some(0));
+        assert_eq!(ef.find(6), Some(3));
+        assert_eq!(ef.find(7), Some(4));
+        assert_eq!(ef.find(10), Some(5));
+        assert_eq!(ef.find(0), None);
+        assert_eq!(ef.find(9), None);
+        assert_eq!(ef.find(100), None);
+
+        assert_eq!(ef.find_range(1..4, 6), Some(3));
+        assert_eq!(ef.find_range(1..3, 6), None);
+        assert_eq!(ef.find_range(4..6, 6), None);
+        assert_eq!(ef.find_range(5..6, 10), Some(5));
+        assert_eq!(ef.find_range(5..5, 10), None);
+    }
+
+    #[test]
+    fn test_random_bits_dense() {
+        for seed in 0..100 {
+            let bits = gen_random_bits(10000, 0.5, seed);
+            let ef = EliasFano::from_bits(&bits, true).unwrap();
+            test_rank_select(&bits, &ef);
+            test_successor_predecessor(&bits, &ef);
+            let queries = gen_random_queries(&bits, 100, seed + 100);
+            test_find(&ef, &queries);
+            let queries = gen_random_range_queries(&bits, 100, seed + 200);
+            test_find_range(&ef, &queries);
+        }
+    }
+
+    #[test]
+    fn test_random_bits_sparse() {
+        for seed in 0..100 {
+            let bits = gen_random_bits(10000, 0.01, seed);
+            let ef = EliasFano::from_bits(&bits, true).unwrap();
+            test_rank_select(&bits, &ef);
+            test_successor_predecessor(&bits, &ef);
+            let queries = gen_random_queries(&bits, 100, seed + 100);
+            test_find(&ef, &queries);
+            let queries = gen_random_range_queries(&bits, 100, seed + 200);
+            test_find_range(&ef, &queries);
+        }
+    }
+
+    #[test]
+    fn test_serialize_dense() {
         let mut bytes = vec![];
-        let ef = EliasFano::from_bits(&gen_random_bits(10000, 42), true).unwrap();
+        let ef = EliasFano::from_bits(&gen_random_bits(10000, 0.5, 42), true).unwrap();
+        let size = ef.serialize_into(&mut bytes).unwrap();
+        let other = EliasFano::deserialize_from(&bytes[..]).unwrap();
+        assert_eq!(ef, other);
+        assert_eq!(size, bytes.len());
+        assert_eq!(size, ef.size_in_bytes());
+    }
+
+    #[test]
+    fn test_serialize_sparse() {
+        let mut bytes = vec![];
+        let ef = EliasFano::from_bits(&gen_random_bits(10000, 0.01, 42), true).unwrap();
         let size = ef.serialize_into(&mut bytes).unwrap();
         let other = EliasFano::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(ef, other);
